// ssa.h
// A sorted sequence stored as an array
#ifndef H_SORTED_SEQ_A
#define H_SORTED_SEQ_A
#define SSA_INIT_SIZE 10

#include <stdlib.h>
#include <string.h>

typedef struct {
  int *data; // the backing array
  size_t cnt;  // the number of elements in the array
  size_t cap;  // the maximum capacity of the array
} sorted_seq_array;


static sorted_seq_array *ssa_create() {
  sorted_seq_array *ssa = (sorted_seq_array *) malloc(sizeof(sorted_seq_array));
  ssa->data = (int *) malloc(sizeof(int) * SSA_INIT_SIZE);
  ssa->cnt = 0;
  ssa->cap = SSA_INIT_SIZE;

  return ssa;
}


static size_t _find_insert_position(int *array, size_t n, int target) {
  int64_t min = 0;
  int64_t max = n-1;

  while (min <= max) {
    size_t mid = (min + max) / 2;
    if (array[mid] < target) {
      min = mid + 1;
    } else {
      max = mid - 1;
    }
  }

  return min;
}


static void ssa_insert(sorted_seq_array *ssa, int x) {
  if (ssa->cnt >= ssa->cap) {
    int *tmp = (int *) realloc(ssa->data, 2*ssa->cap*sizeof(int));
    ssa->data = tmp;
    ssa->cap = 2*ssa->cap;
  }

  size_t idx = _find_insert_position(ssa->data, ssa->cnt, x);

  if (idx < ssa->cnt)  {
    memmove(ssa->data + idx + 1, ssa->data + idx, (ssa->cnt - idx) * sizeof(int));
  }

  ssa->data[idx] = x;
  ssa->cnt++;
}


static ssize_t ssa_search(sorted_seq_array *ssa, int x) {
  size_t idx = _find_insert_position(ssa->data, ssa->cnt, x);

  if (ssa->data[idx] == x) {
    return idx;
  }

  return -1;
}


static void ssa_destroy(sorted_seq_array *asa) {
  free(asa->data);
  free(asa);
}
#endif
